@@ -11,11 +11,14 @@ from rest_framework.compat import unicode_http_header
 from rest_framework.utils.mediatypes import _MediaType
 
 from django_datadog_logger.encoders import SafeJsonEncoder
+from django_datadog_logger.celery import get_task_name
 import django_datadog_logger.celery
 import django_datadog_logger.wsgi
 
 # those fields are excluded from extra dict
 # and remains acceptable in record
+from django_datadog_logger.recursion import not_recursive, RecursionDetected
+
 EXCLUDE_FROM_EXTRA_ATTRS = {
     "user",
     "auth",
@@ -45,6 +48,13 @@ def determine_version(request):
     return version or None
 
 
+@not_recursive
+def get_wsgi_request_user(wsgi_request):
+    if getattr(wsgi_request, "user", None) is not None:
+        if getattr(wsgi_request.user, "is_authenticated", False):
+            return wsgi_request.user
+
+
 class DataDogJSONFormatter(json_log_formatter.JSONFormatter):
     def json_record(self, message, extra, record):
 
@@ -57,13 +67,14 @@ class DataDogJSONFormatter(json_log_formatter.JSONFormatter):
             "syslog.severity": record.levelname,
         }
 
+        # Add special `dd.` log record attributes added by `ddtrace` library
+        # For example: dd.trace_id, dd.span_id, dd.service, dd.env, dd.version, etc
+        log_entry_dict.update(self.get_datadog_attributes(record))
+
         celery_request = self.get_celery_request(record)
         if celery_request is not None:
-            log_entry_dict["celery.request_id"] = celery_request.id
-            if isinstance(celery_request.task, str):
-                log_entry_dict["celery.task_name"] = celery_request.task
-            elif hasattr(celery_request.task, "name"):
-                log_entry_dict["celery.task_name"] = celery_request.task.name
+            log_entry_dict["celery.request_id"] = celery_request["id"]
+            log_entry_dict["celery.task_name"] = celery_request["name"]
 
         wsgi_request = self.get_wsgi_request()
         if wsgi_request is not None:
@@ -92,14 +103,15 @@ class DataDogJSONFormatter(json_log_formatter.JSONFormatter):
             ):
                 log_entry_dict["usr.session_id"] = wsgi_request.auth["sid"]
 
-            if getattr(wsgi_request, "user", None) is not None and getattr(
-                wsgi_request.user, "is_authenticated", False
-            ):
-                log_entry_dict["usr.id"] = getattr(wsgi_request.user, "pk", None)
-                log_entry_dict["usr.name"] = getattr(
-                    wsgi_request.user, getattr(wsgi_request.user, "USERNAME_FIELD", "username"), None
-                )
-                log_entry_dict["usr.email"] = getattr(wsgi_request.user, "email", None)
+            try:
+                user = get_wsgi_request_user(wsgi_request)
+            except RecursionDetected:
+                user = None
+
+            if user:
+                log_entry_dict["usr.id"] = getattr(user, "pk", None)
+                log_entry_dict["usr.name"] = getattr(user, getattr(user, "USERNAME_FIELD", "username"), None)
+                log_entry_dict["usr.email"] = getattr(user, "email", None)
 
             if getattr(wsgi_request, "session", None) is not None and getattr(wsgi_request.session, "session_key"):
                 log_entry_dict["usr.session_key"] = wsgi_request.session.session_key
@@ -139,10 +151,17 @@ class DataDogJSONFormatter(json_log_formatter.JSONFormatter):
         return log_entry_dict
 
     def get_celery_request(self, record):
-        if record.name == "celery.worker.strategy" and record.args and isinstance(record.args[0], Request):
-            return record.args[0]
+        if record.name == "celery.worker.strategy" and record.args:
+            if isinstance(record.args, (list, tuple)) and isinstance(record.args[0], Request):
+                return {"id": record.args[0].id, "name": get_task_name(record.args[0])}
+            elif isinstance(record.args, dict):
+                return {"id": record.data.get("id"), "name": record.data.get("name")}
         return django_datadog_logger.celery.get_celery_request()
 
+    def get_datadog_attributes(self, record):
+        """Helper to extract dd.* attributes from the log record."""
+        return {attr_name: record.__dict__[attr_name] for attr_name in record.__dict__ if attr_name.startswith("dd.")}
+
     def get_wsgi_request(self):
         return django_datadog_logger.wsgi.get_wsgi_request()
 
